*(SOUNDEX.CMD                                by Paul Litwin   06/26/90 )
*(For R:BASE version 2.11.                                             )
*(This routine converts a surname stored in the global variable vlname )
*(to its Russell soundex code, stored in the variable vsdx.            )

*(Global input variables:                                              )
*(    vlname   = name to be processed.                                 )
*(    vlen     = length of name to be processed.                       )

*(Local variables:                      .                              )
*(    vchar    = current character in name being processed.            )
*(    vcno     = position of current character in name.                )
*(    vsound1  = soundex number for current character.                 )
*(    vsound2  = soundex number for previous character.                )
*(    vsep     = indicates whether previous letter was a vowel to act  )
*(               as a seperator character for double consonants.       )
*(    vscount  = counts the number of characters in the soundex code,  )
*(               when equal to 4, we are finished.                     )

*(Global variables returned:                                           )
*(    vsdx     = soundex code for name.                                )

SET QUOTE="

SET V vchar   TEXT    +
      vcno    INTEGER +
      vsound1 TEXT    +
      vsound2 TEXT    +
      vsep    INTEGER +
      vscount INTEGER +
      vsdx    TEXT 

WRITE "Standby while creating Soundex code..." AT 1,40

*(Initialize several variables.                                        )
SET V vsep    TO 0 +
      vscount TO 0 +
      vcno    TO 0

*(Loop through each character of name until all four characters of     )
*(soundex code have been built.                                        )
WHILE vscount < 4 THEN

*(Increment index variable for last name.                              )
  SET V vcno TO (.vcno + 1)

*(If vcno is > length of name, then we have processed all of the name  )
*(and must add 0's to soundex code.                                    )
  IF vcno > .vlen THEN   
    SET V vsdx    TO (.vsdx + "0") +
          vscount TO (.vscount + 1)
    GOTO skip1
  ENDIF

*(Set vchar to next character in name.                                 )
  SET V vchar TO (sget(.vlname,1,.vcno))

*(Initialize vsound1 to a blank.                                       )
  SET V vsound1 TO " "

*(Now classify the character and set vsound1 to the numeric code       )
*(corresponding to its class:                                          )
*( -If the letter is W or H, ignore it.                                )
*( -If a vowel or Y, assign it a 0 code and set vsep to 1.             )
*( -Otherwise, assign it a code from 1 to 6.                           )
*(Convert the numberic code to a text string that can be added to vsdx.)

  IF vchar = "A" OR vchar = "E" OR vchar = "I" OR vchar = "O" OR +
     vchar = "U" OR vchar = "Y" THEN
   SET V vsound1 TO (ctxt(0)) +
         vsep TO 1
  ENDIF

  IF vchar = "B" OR vchar = "F" OR vchar = "P" OR vchar = "V" THEN
    SET V vsound1 TO (ctxt(1))
  ENDIF

  IF vchar = "C" OR vchar = "G" OR vchar = "J" OR vchar = "K" OR +
     vchar = "Q" OR vchar = "S" OR vchar = "X" OR vchar = "Z" THEN
     SET V vsound1 TO (ctxt(2))
  ENDIF

  IF vchar = "D" OR vchar = "T" THEN
     SET V vsound1 TO (ctxt(3))
  ENDIF

  IF vchar = "L" THEN
    SET V vsound1 TO (ctxt(4))
  ENDIF

  IF vchar = "M" OR vchar = "N" THEN
    SET V vsound1 TO (ctxt(5))
  ENDIF

  IF vchar = "R" THEN  
    SET V vsound1 TO (ctxt(6))
  ENDIF

*(If this is the first character, don't use the code.  Use the actual  )
*(letter instead.  But it is important to still calculate the code and )
*(store its value in vsound2 for the checking of repeat consonants.    )
  IF vcno = 1 THEN
    SET V vsdx    TO (luc(.vchar))  +
          vscount TO (.vscount + 1) +
          vsound2 TO (.vsound1)
    GOTO skip1
  ENDIF

*(If a significant character -- a consonant other than W or H -- and if)
*(the character is not of the same code as the previous one, or if it  )
*(is of the same code, but a vowel separates the two consonants, then  )
*(add the code stored in vsound1 to vsdx.                              )
  IF vsound1 <> " "  AND vsound1 <> "0" THEN 

    IF vsound1 <> .vsound2 OR vsep = 1 THEN 
      SET V vsdx    TO (.vsdx + .vsound1) +
            vscount TO (.vscount + 1)     +
            vsound2 TO (.vsound1)         +
            vsep    TO 0
    ENDIF *(IF vsound1 <> .vsound2 OR...)

  ENDIF *(IF vsound1 <> " " AND...)

  LABEL skip1
ENDWHILE

*(Clean up time.                                                       )
CLEAR vsound1 +
      vsound2 +
      vsep +
      vscount +
      vcno +
      vchar 

CLS FROM 1 TO 1

RETURN
